/* global browser, app, Port, _, cfg */
"use strict";

// Older Gecko versions open a pop-up window instead of a content tab
if (!location.hash && app.platform_version < 7) {
	Port.send({
		"cmd": "open",
		"url": "options.html#general"
	});
	window.close();
}

var def_prefs, input_changes = {},
	$ = function(id) {
		return document.getElementById(id);
	},
	processLNG = function(nodes) {
		var i = nodes.length, els, l, args, string;

		while (i--) {
			if (nodes[i].lng_loaded) {
				continue;
			}

			els = nodes[i].querySelectorAll("[data-lng]");
			l = els.length;

			while (l--) {
				string = _(els[l].dataset["lng"]);

				if (els[l].dataset["lngargs"]) {
					args = els[l].dataset["lngargs"].split(" ");
					args.idx = args.length;

					while (args.idx--) {
						string = string.replace(
							"%" + args[args.idx],
							els[l].dataset[args[args.idx]] || args[args.idx]
						);
					}
				}

				els[l][els[l].dataset["lngattr"] || "innerHTML"] = string;
			}

			nodes[i].lng_loaded = true;
		}
	};

function color_trans(node, color, time) {
	clearTimeout(node.col_trans_timer);

	if (color === null) {
		node.style.color = "";
		delete node.col_trans_timer;
		return;
	}

	node.style.color = color;
	node.col_trans_timer = setTimeout(function() {
		color_trans(node, null);
	}, time || 2000);
}

function fill_output(e) {
	e = e.target || e;
	var op = e.previousElementSibling;
	op.value = op.dataset["as_percent"] ? parseInt(e.value * 100, 10) : e.value;
	op.defaultValue = def_prefs[e.name];
}

function color_text_input(e) {
	e = e.type === "input" ? this : e;
	var v = /^#([\da-f]{3}){1,2}/i.test(e.value) ? e.value : "#ffffff";
	e.previousElementSibling.value = v.length === 4 ? "#" + v[1] + v[1] + v[2] + v[2] + v[3] + v[3]: v;
}

function color_change() {
	this.nextElementSibling.value = this.value;
}

function setDefault(query) {
	if (!query) {
		return;
	}

	[].forEach.call(typeof query === "string" ? document.querySelectorAll(query) : [query], function(el) {
		if (el.type === "checkbox") {
			el.checked = el.defaultChecked;
		}
		else if (/^SELECT/i.test(el.type)) {
			for (var i = el.length; i--;) {
				if (el[i].hasAttribute("selected")) {
					el.selectedIndex = i;
					break;
				}
			}
		}
		else {
			el.value = el.defaultValue;

			if (el.type === "range") {
				fill_output(el);
			}
		}
	});
}

function load(prefs) {
	if (!prefs) {
		cfg.getItem("cfg", function(value) {
			load(JSON.parse(value) || {});
		});

		return;
	}

	var
		fields = document.querySelectorAll("[name]"),
		i = fields.length,
		m, fld, fld_type, pref;

	def_prefs = new XMLHttpRequest;
	def_prefs.overrideMimeType("application/json;charset=utf-8");
	def_prefs.open("GET", "defaults.jsn", false);
	def_prefs.send(null);
	def_prefs = JSON.parse(def_prefs.responseText);

	while (i--) {
		fld = fields[i];
		pref = fld.name;

		if (fld.disabled || fld.readOnly || def_prefs[pref] === void 0) {
			continue;
		}

		fld_type = fld.getAttribute("type") || "text";

		if (fld.type !== fld_type) {
			fld_type = fld.type;
		}

		if (fld_type === "checkbox") {
			fld.checked = fld.defChecked = !!(prefs[pref] === void 0 ? def_prefs : prefs)[pref];
			fld.defaultChecked = def_prefs[pref];
		}
		else {
			if (fld_type.slice(0, 6) === "select") {
				[].some.call(fld, function(el) {
					if (el.value == def_prefs[pref]) {
						el.defaultSelected = true;
						return true;
					}
				});
			}
			else {
				fld.defaultValue = def_prefs[pref];
			}

			fld.value = fld.defValue = (prefs[pref] === void 0 ? def_prefs : prefs)[pref];

			if (fld_type === "range") {
				m = fld.previousElementSibling;

				if (m && m.nodeName === "OUTPUT") {
					fill_output(fld);
				}

				m = m.previousElementSibling;

				if (m && m.getAttribute("type") === "color") {
					m.style.opacity = fld.value;
				}

				fld.addEventListener("change", fill_output, false);
			}
			else if (fld_type === "text" && fld.previousElementSibling
				&& fld.previousElementSibling.getAttribute("type") === "color") {
				fld.addEventListener("input", color_text_input, false);
				color_text_input(fld);
				fld.previousElementSibling.addEventListener("change", color_change, false);
			}
		}
	}
}

function save() {
	var i, m, fld, fld_type,
		fields = document.querySelectorAll("[name]"),
		pref, prefs = {};

	for (i = 0; i < fields.length; ++i) {
		fld = fields[i];
		pref = fld.name;

		if (fld.disabled || fld.readOnly || def_prefs[pref] === void 0) {
			continue;
		}

		fld_type = fld.getAttribute("type");

		if (fld_type === "checkbox") {
			prefs[pref] = fld.checked;
		}
		else if (fld_type === "range" || fld_type === "number" || fld.classList.contains("number")) {
			prefs[pref] = (fld.min ? Math.max(fld.min, Math.min(fld.max, parseFloat(fld.value))) : parseFloat(fld.value));

			if (typeof prefs[pref] !== "number") {
				prefs[pref] = parseFloat(fld.defaultValue);
			}

			fld.value = prefs[pref];
		}
		else {
			prefs[pref] = fld.value;
		}

		if (prefs[pref] === void 0) {
			prefs[pref] = def_prefs[pref];
		}
	}

	Port.send({"cmd": "save", "prefs": prefs});
}

window.onhashchange = function() {
	var section, args = [],
		menu = $("nav_menu"),
		old = menu && menu.active && menu.active.hash.slice(1) || "general",
		hash = location.hash.slice(1) || "general";

	if (hash.indexOf("/") > -1) {
		args = hash.split("/");
		hash = args.shift();
	}

	section = $(hash + "_sec");

	if (!section.lng_loaded) {
		if (hash === "info") {
			if (args[0]) {
				$(args[0] === "0" ? "app_installed" : "app_updated").style.display = "block";
			}

			if (!_.locales) {
				_.locales = new XMLHttpRequest;
				_.locales.overrideMimeType("application/json;charset=utf-8");
				_.locales.open("GET", "./locales.jsn", false);
				_.locales.send(null);
				_.locales = _.locales.responseText ? JSON.parse(_.locales.responseText) : [];
			}

			var locales = "";
			var lng_map = function(el) {
				el.name = ((el.name || el.fullname || "") + (el.fullname && el.name ? " (" + el.fullname + ")" : "")) || el.email || el.web;

				return el.email || el.web
					? '<a href="' + (el.email ? "mailto:" + el.email : el.web) + '">' + el.name + "</a>"
					: el.name;
			};

			for (var alpha2 in _.locales) {
				if (alpha2 === "_") {
					continue;
				}

				locales += '<tr>' +
					"<td" + (_.locales[alpha2]['%'] ? ' title="' + _.locales[alpha2]['%'] + '%"' : "") + ">" +
						alpha2 + ", " + _.locales[alpha2].name + "</td>" +
					"<td>" + (_.locales[alpha2].translators ?
						_.locales[alpha2].translators.map(lng_map).join(", ")
						: "anonymous") +
					"</td></tr>";
			}

			$("locales_table").innerHTML += locales;
		}
	}

	if (old !== hash && (old = $(old + "_sec"))) {
		old.style.display = "none";
	}

	if (section) {
		processLNG([section]);
		section.style.display = "block";
	}

	if (menu.active) {
		menu.active.classList.remove("active");
	}

	if (menu.active = menu.querySelector('a[href="#' + hash + '"]')) {
		menu.active.classList.add("active");
	}
};

window.addEventListener("load", function() {
	var tmp = $("app_version");
	tmp.textContent = app.name + " v" + app.version;

	["opera", "firefox", "chrome", "safari", "maxthon"].some(function(el) {
		if (browser[el]) {
			document.body.classList.add(el);
			return true;
		}

		return false;
	});

	var menu = $("nav_menu");

	processLNG([menu, $("right_panel").firstElementChild]);

	load();

	tmp = document.body.querySelector('.color_helper>input[type="text"]');

	if (tmp) {
		tmp.addEventListener("input", color_text_input, false);
		color_text_input(tmp);
		tmp.previousElementSibling.addEventListener("change", color_change, false);
	}

	menu.onclick = function(e) {
		if (e.target.hash) {
			e.preventDefault();
			location.hash = e.target.hash;
		}
	};

	window.onhashchange();

	document.body.style.display = "block";

	document.forms[0].addEventListener("keydown", function(e) {
		e.stopPropagation();

		if (e.which === 13) {
			e.target.form_saved = true;
		}

		if (e.repeat || !e.target.name || e.target.name.indexOf("key_") !== 0
			|| e.ctrlKey || e.altKey || e.metaKey
			|| e.which < 47) {
			return;
		}

		e.preventDefault();
		color_trans(e.target, null);

		var keys = {
				96: "0", 97: "1", 98: "2", 99: "3", 100: "4",
				101: "5", 102: "6", 103: "7", 104: "8", 105: "9",
				106: "*", 107: "+", 109: "-", 110: ".", 111: "/",
				186: ";", 187: "=", 188: ",", 189: "-", 190: ".",
				191: "/", 192: "`", 219: "[", 220: "\\", 221: "]", 222: "'",
				112: "F1", 113: "F2", 114: "F3", 115: "F4", 116: "F5", 117: "F6",
				118: "F7", 119: "F8", 120: "F9", 121: "F10", 122: "F11", 123: "F12"
			},
			key = keys[e.which] || String.fromCharCode(e.which).toUpperCase();

		keys = document.body.querySelectorAll('input[name^="key_"]');

		for (var i = 0; i < keys.length; ++i) {
			if (keys[i].value.toUpperCase() === key) {
				if (e.target !== keys[i]) {
					color_trans(e.target, "red");
				}

				return false;
			}
		}

		e.target.value = key;
		document.forms[0].onchange(e);
	}, false);

	document.forms[0].addEventListener("contextmenu", function(e) {
		e.stopPropagation();
		var t = e.target;

		if (t.classList.contains("checkbox")) {
			t = t.previousElementSibling;
		}

		if (!t.name) {
			return;
		}

		if (e.ctrlKey) {
			e.preventDefault();
			setDefault(t);
			document.forms[0].onchange({target: t});
		}
	}, false);

	// leave it callable
	document.forms[0].onchange = function(e) {
		if (e.stopPropagation) {
			e.stopPropagation();
		}

		var defval, t = e.target;

		if (t.form_saved) {
			delete t.form_saved;
		}
		else if (t.parentNode.dataset["form"]
			|| t.parentNode.parentNode.dataset["form"]) {
			defval = "default";
		}
		else if (t.name.indexOf("_") > 0) {
			defval = "def";
		}

		if (!defval) {
			return;
		}

		if ((t.type === "checkbox" && t[defval + "Checked"] !== t.checked)
			|| (t.type !== "checkbox" && t[defval + "Value"] != t.value)) { // != is not a mistake
			input_changes[t.name] = true;
		}
		else {
			delete input_changes[t.name];
		}

		$("save_button").style.color = Object.keys(input_changes).length ? "#e03c00" : "";
	};

	var reset_button = $("reset_button");

	reset_button.reset = function() {
		delete reset_button.pending;
		reset_button.style.color = "#000";
	};

	reset_button.addEventListener("click", function(e) {
		if (reset_button.pending) {
			input_changes["form_reset"] = true;
			clearTimeout(reset_button.pending);
			delete this.pending;
			reset_button.style.color = "";
			reset_button.nextElementSibling.style.color = "#e03c00";

			if (e.ctrlKey) {
				e.preventDefault();
				e = (location.hash || "#general") + "_sec ";
				setDefault(e + "input," + e + "select," + e + "textarea");
				reset_button.style.color = "lime";
			}
			else {
				reset_button.style.color = "green";
			}

			reset_button.pending = setTimeout(reset_button.reset, 2000);

			return;
		}

		reset_button.style.color = "orange";
		reset_button.pending = setTimeout(reset_button.reset, 2000);
		e.preventDefault();
	}, false);

	$("save_button").addEventListener("click", function(e) {
		e.preventDefault();
		save();
		color_trans(this, "green");
	}, false);

	[].forEach.call(document.body.querySelectorAll(".action_buttons") || [], function(el) {
		el.onmousedown = function(e) {
			e.preventDefault();
		};
	});
}, false);